.include "nes.h"
.export clearLineImg, putTile, copyLineImg, clearMap, lineImgBuf, vwfPuts
.import chrData

.segment "BSS"
.align 256
lineImgBuf: .res 256

.segment "CODE"


;
; Clears the line image buffer.
;
clearLineImg:
  ldx #$77
  lda #0
:
  sta lineImgBuf+$08,x
  sta lineImgBuf+$80,x
  dex
  bpl :-

  ; Make opaque columns for sprite 0
  ldx #$07
:
  lda #$F0
  sta lineImgBuf+$00,x
  lda #$0F
  sta lineImgBuf+$F8,x
  dex
  bpl :-
  rts

;
; Puts a tile to position X.
; In:   A = tile number
;       X = destination X position
; Trash: 11-15
putTile:
@curHiBits = 11
@dstAddr = 12
@nShifts = 13
@tileAddr = 14
  ; Find source address
  sta @tileAddr
  lda #0
  asl @tileAddr
  rol a
  asl @tileAddr
  rol a
  asl @tileAddr
  rol a
  adc #>chrData
  sta @tileAddr+1

  ; Find destination address
  txa
  and #%00000111
  sta @nShifts
  txa
  and #%11111000
  sta @dstAddr
  ldy #0
@loop:
  lda #0
  sta @curHiBits
  lda (@tileAddr),y

  ; Shift byte into position
  ldx @nShifts
  beq @notShifted
:
  lsr a
  ror @curHiBits
  dex
  bne :-
@notShifted:

  ; OR byte into destination
  ldx @dstAddr
  ora lineImgBuf,x
  sta lineImgBuf,x
  lda @curHiBits
  ora lineImgBuf+8,x
  sta lineImgBuf+8,x
  inc @dstAddr
  iny
  cpy #8
  bcc @loop
  rts


;
; Copies the line to the screen
; in:  PPUADDR = destination address
;
copyLineImg:
  .repeat 16,chr
    .repeat 2,bank
      .repeat 8,scanline
        lda lineImgBuf + scanline + bank*128 + chr*8
        sta PPUDATA
      .endrep
    .endrep
  .endrep
  rts


;
; Clears a nametable for use with VWF.
; in:  PPUADDR = destination address
;
clearMap:
@nLines = 14
@base = 15

  lda #30
  sta @nLines
  lda #0
  sta @base
@loop1:
  ldx @base
  ldy #16
:
  stx PPUDATA
  inx
  dey
  bne :-
  ldx @base
  ldy #16
:
  stx PPUDATA
  inx
  dey
  bne :-
  lda @base
  clc
  adc #16
  sta @base
  dec @nLines
  bne @loop1

  ; Set up attributes
  ldy #8
  lda #0
  ldx #$55
:
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  stx PPUDATA
  stx PPUDATA
  stx PPUDATA
  stx PPUDATA
  dey
  bne :-
  rts


;
; Puts a tile to position X.
; In:   AAYY = string base address
;       X = destination X position
; Trash: 8-15
;
vwfPuts:
@str = 8
@horz = 10
  sty @str
  sta @str+1
  stx @horz
@loop:
  ldy #0
  lda (@str),y
  beq @done
  ldx @horz
  jsr putTile
  ldy #0
  lda (@str),y
  tax
  lda chrWidths,x
  clc
  adc @horz
  sta @horz
  cmp #248
  bcs @done
  inc @str
  bne @loop
  inc @str+1
  bne @loop

@done:
  rts

.segment "RODATA"
.align 256
chrWidths:
  .incbin "widths.bin"
